from pathlib import Path

import polars as pl

from pitchoune.io import IO


class CSV_IO(IO):
    """CSV IO class for reading and writing CSV files using Polars."""
    def __init__(self):
        super().__init__(suffix="csv")

    def deserialize(self, filepath: Path|str, schema=None, separator: str=";", decimal_comma: bool = False, encoding: str="utf-8", **params) -> None:
        """Read a CSV file and return a Polars DataFrame."""
        return pl.read_csv(str(filepath), schema_overrides=schema, encoding=encoding, separator=separator, decimal_comma=decimal_comma, **params)

    def serialize(self, df: pl.DataFrame, filepath: Path|str, separator: str=";", **params) -> None:
        """Write a Polars DataFrame to a CSV file."""
        df.write_csv(str(filepath), separator=separator, quote_style="non_numeric", include_bom=True, **params)



from pitchoune.factory import Factory
from pitchoune.io import IO
from pitchoune.ios.csv_io import CSV_IO
from pitchoune.ios.jsonl_io import JSONL_IO
from pitchoune.ios.tsv_io import TSV_IO
from pitchoune.ios.xlsx_io import XLSX_IO
from pitchoune.ios.xlsm_io import XLSM_IO


class IOFactory(Factory):
    """Factory class to create deserializer instances."""
    def __init__(self):
        self._ios = {
            "jsonl": JSONL_IO,
            "xlsx": XLSX_IO,
            "csv": CSV_IO,
            "tsv": TSV_IO,
            "xlsm": XLSM_IO
        }
        super().__init__(base_class=IO)

    def create(self, *args, suffix: str, **kwargs):
        """Create an instance of the specified IO class."""
        if suffix not in self._ios:
            raise ValueError(f"Unsupported file type: {suffix}. Supported types are: {', '.join(self._ios.keys())}")
        cls = self._ios[suffix]
        return super().create(cls, *args, **kwargs)
    
    def register(self, suffix: str, cls: IO):
        """Register a new IO class with a specific suffix."""
        self._ios[suffix] = cls



class IO:
    """Base class for IO operations."""
    def __init__(self, suffix: str):
        self._suffix = suffix

    @property
    def suffix(self) -> str:
        return self._suffix
    
    def serialize(self, *args, **kwargs) -> None:
        """Serialize data to a file."""
        pass

    def deserialize(self, *args, **kwargs) -> None:
        """Deserialize data from a file."""
        pass



from pathlib import Path
from typing import Any

import polars as pl

from pitchoune.io import IO


class XLSX_IO(IO):
    """XLSX IO class for reading and writing XLSX files using Polars."""
    def __init__(self):
        super().__init__(suffix="xlsx")

    def deserialize(self, filepath: Path|str, schema=None, sheet_name: str = "sheet1", engine: str = "openpyxl", read_options: dict[str, Any] = None, **params) -> None:
        """Read an XLSX file and return a Polars DataFrame."""
        return pl.read_excel(
            str(filepath),
            schema_overrides=schema,
            sheet_name=sheet_name,
            engine=engine,
            read_options=read_options,
            infer_schema_length=10000,
            **params
        )

    def serialize(self, df: pl.DataFrame, filepath: Path|str, **params) -> None:
        """Write a Polars DataFrame to an XLSX file."""
        df.write_excel(str(filepath), **params)
